/*
 * Copyright (C) 2013 Marten Gajda <marten@dmfs.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

package org.dmfs.tasks.model.adapters;

import org.dmfs.tasks.model.ContentSet;
import org.dmfs.tasks.model.OnContentChangeListener;

import android.database.Cursor;


/**
 * Knows how to load and store a {@link Boolean} value in a certain field of a {@link ContentSet}.
 * 
 * @author Marten Gajda <marten@dmfs.org>
 */
public final class BooleanFieldAdapter extends FieldAdapter<Boolean>
{

	/**
	 * The field name this adapter uses to store the values.
	 */
	private final String mFieldName;

	/**
	 * The default value, if any.
	 */
	private final Boolean mDefaultValue;


	/**
	 * Constructor for a new IntegerFieldAdapter without default value.
	 * 
	 * @param fieldName
	 *            The name of the field to use when loading or storing the value.
	 */
	public BooleanFieldAdapter(String fieldName)
	{
		if (fieldName == null)
		{
			throw new IllegalArgumentException("fieldName must not be null");
		}
		mFieldName = fieldName;
		mDefaultValue = null;
	}


	/**
	 * Constructor for a new BooleanFieldAdapter with default value.
	 * 
	 * @param fieldName
	 *            The name of the field to use when loading or storing the value.
	 * @param defaultValue
	 *            The default value.
	 */
	public BooleanFieldAdapter(String fieldName, Boolean defaultValue)
	{
		if (fieldName == null)
		{
			throw new IllegalArgumentException("fieldName must not be null");
		}
		mFieldName = fieldName;
		mDefaultValue = defaultValue;
	}


	@Override
	public Boolean get(ContentSet values)
	{
		Integer value = values.getAsInteger(mFieldName);

		return value != null && value > 0;
	}


	@Override
	public Boolean get(Cursor cursor)
	{
		int columnIdx = cursor.getColumnIndex(mFieldName);
		if (columnIdx < 0)
		{
			throw new IllegalArgumentException("The fieldName column missing in cursor.");
		}
		return !cursor.isNull(columnIdx) && cursor.getInt(columnIdx) > 0;
	}


	@Override
	public Boolean getDefault(ContentSet values)
	{
		return mDefaultValue;
	}


	@Override
	public void set(ContentSet values, Boolean value)
	{
		values.put(mFieldName, value ? 1 : 0);
	}


	@Override
	public void registerListener(ContentSet values, OnContentChangeListener listener, boolean initalNotification)
	{
		values.addOnChangeListener(listener, mFieldName, initalNotification);
	}


	@Override
	public void unregisterListener(ContentSet values, OnContentChangeListener listener)
	{
		values.removeOnChangeListener(listener, mFieldName);
	}
}
